from glad.lang.c.loader.egl import EGLCLoader as EGLLoader
from glad.lang.c.loader.gl import OpenGLCLoader as GLLoader
from glad.lang.c.loader.glx import GLXCLoader as GLXLoader
from glad.lang.c.loader.wgl import WGLCLoader as WGLLoader

from glad.lang.c.generator import CGenerator as Generator
