from glad.lang.volt.loader.egl import EGLVoltLoader as EGLLoader
from glad.lang.volt.loader.gl import OpenGLVoltLoader as GLLoader
from glad.lang.volt.loader.glx import GLXVoltLoader as GLXLoader
from glad.lang.volt.loader.wgl import WGLVoltLoader as WGLLoader

from glad.lang.volt.generator import VoltGenerator as Generator