from glad.lang.d.loader.egl import EGLDLoader as EGLLoader
from glad.lang.d.loader.gl import OpenGLDLoader as GLLoader
from glad.lang.d.loader.glx import GLXDLoader as GLXLoader
from glad.lang.d.loader.wgl import WGLDLoader as WGLLoader

from glad.lang.d.generator import DGenerator as Generator